"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const precise_proofs_js_1 = require("precise-proofs-js");
const origin_backend_core_1 = require("@energyweb/origin-backend-core");
class PreciseProofEntity {
    constructor(id, configuration) {
        this.id = id;
        if (!configuration.offChainDataSource) {
            throw new Error('Entity::constructor: Please set offChainDataSource in the configuration.');
        }
        this.configuration = configuration;
        this.proofs = [];
    }
    get certificateClient() {
        return this.configuration.offChainDataSource.certificateClient;
    }
    async saveCommitment(proof) {
        const commitmentStatus = await this.certificateClient.addOwnershipCommitment(this.id, proof);
        if (this.configuration.logger) {
            if (commitmentStatus === origin_backend_core_1.CommitmentStatus.REJECTED) {
                this.configuration.logger.error('Unable to save the commitment. Rejected.');
            }
            else if (commitmentStatus === origin_backend_core_1.CommitmentStatus.CURRENT) {
                this.configuration.logger.verbose(`Commitment saved to for Certificate #${this.id}`);
            }
        }
        return commitmentStatus;
    }
    async getCommitment() {
        const proof = await this.certificateClient.getOwnershipCommitment(this.id);
        if (!proof) {
            throw new Error('getCommitment(): Not found.');
        }
        this.generateAndAddProofs(proof.commitment, proof.salts);
        this.verifyOffChainProperties(this.propertiesDocumentHash, proof.commitment);
        if (this.configuration.logger) {
            this.configuration.logger.verbose(`Got commitment for Certificate #${this.id}`);
        }
        return proof;
    }
    async getPendingTransferCommitment() {
        const proof = await this.certificateClient.getPendingOwnershipCommitment(this.id);
        if (!proof) {
            throw new Error('getCommitment(): Not found.');
        }
        if (this.configuration.logger) {
            this.configuration.logger.verbose(`Got pending transfer commitment for Certificate #${this.id}`);
        }
        return proof;
    }
    verifyOffChainProperties(rootHash, properties) {
        Object.keys(properties).map((key) => {
            const theProof = this.proofs.find((proof) => proof.key === key);
            if (this.configuration.logger) {
                this.configuration.logger.debug('\nDEBUG verifyOffChainProperties');
                this.configuration.logger.debug(`rootHash: ${rootHash}`);
                this.configuration.logger.debug(`properties: ${properties}`);
            }
            if (theProof) {
                if (!precise_proofs_js_1.PreciseProofs.verifyProof(rootHash, theProof)) {
                    throw new Error(`Proof ${JSON.stringify(theProof)} for property ${key} is invalid.`);
                }
            }
            else {
                throw new Error(`Could not find proof for property ${key}`);
            }
            return true;
        });
    }
    generateAndAddProofs(commitment, salts) {
        this.proofs = [];
        let leafs = salts
            ? precise_proofs_js_1.PreciseProofs.createLeafs(commitment, salts)
            : precise_proofs_js_1.PreciseProofs.createLeafs(commitment);
        leafs = precise_proofs_js_1.PreciseProofs.sortLeafsByKey(leafs);
        const merkleTree = precise_proofs_js_1.PreciseProofs.createMerkleTree(leafs.map((leaf) => leaf.hash));
        leafs.forEach((leaf) => this.addProof(precise_proofs_js_1.PreciseProofs.createProof(leaf.key, leafs, false)));
        const result = {
            commitment,
            rootHash: precise_proofs_js_1.PreciseProofs.getRootHash(merkleTree),
            salts: leafs.map((leaf) => leaf.salt),
            leafs
        };
        if (this.configuration.logger) {
            this.configuration.logger.debug('\nDEBUG generateAndAddProofs');
            this.configuration.logger.debug(result);
            precise_proofs_js_1.PreciseProofs.printTree(merkleTree, leafs);
        }
        return result;
    }
    addProof(proof) {
        this.proofs.push(proof);
    }
}
exports.PreciseProofEntity = PreciseProofEntity;
//# sourceMappingURL=PreciseProofEntity.js.map